@@ -1,5 +1,195 @@
 import numpy as np
 import json
+import copy
+from sklearn.metrics import cohen_kappa_score
+
+def load_data(data_path, require_annotations=True):
+    data_candidates = json.load(open(data_path))
+    if require_annotations:
+        data = {'projects': []}
+        # Keep only projects that have annotations
+        for project in data_candidates['projects']:
+            keep = False
+            for document in project['documents']:
+                if len(document['annotations']) > 0:
+                    keep = True
+                    break
+            if keep:
+                data['projects'].append(project)
+    else:
+        data = data_candidates
+
+    return data
+
+
+def get_doc_from_project(project, doc_id):
+    for document in project['documents']:
+        if document['id'] == doc_id:
+            return document
+    return None
+
+
+def get_ann_from_doc(document, start, end):
+    for ann in document['annotations']:
+        if ann['start'] == start and ann['end'] == end:
+            return ann
+    return None
+
+
+def meta_ann_from_ann(ann, meta_name):
+    meta_anns = ann['meta_anns']
+    # need for old versions of data
+    if type(meta_anns) == dict:
+        return meta_anns.get(meta_name, Name)
+    else:
+        for meta_ann in meta_anns:
+            if meta_ann['name'] == meta_name:
+                return meta_ann
+    return None
+
+
+def are_anns_same(ann, ann2, meta_names=[], require_double_inner=True):
+    if ann['cui'] == ann2['cui'] and \
+       ann['correct'] == ann2['correct'] and \
+       ann['deleted'] == ann2['deleted'] and \
+       ann['alternative'] == ann2['alternative'] and \
+       ann['killed'] == ann2['killed'] and \
+       ann['manually_created'] == ann2['manually_created'] and \
+       ann['validated'] == ann2['validated']:
+        #Check are meta anns the same if exist
+        for meta_name in meta_names:
+            meta_ann = meta_ann_from_ann(ann, meta_name)
+            meta_ann2 = meta_ann_from_ann(ann2, meta_name)
+            if meta_ann is not None and meta_ann2 is not None:
+                if meta_ann['value'] != meta_ann2['value']:
+                    return False
+            elif (meta_ann is None) != (meta_ann2 is None):
+                if require_double_inner:
+                    return False
+    else:
+        return False
+
+    return True
+
+
+def are_docs_same(document, document2, require_double_inner=True, ann_stats=[], meta_names=[]):
+    are_same = True
+
+    for ann in document['annotations']:
+        # Take only validated anns
+        if ann['validated']:
+            ann2 = get_ann_from_doc(document2, ann['start'], ann['end'])
+
+            if ann2 is not None:
+                if not are_anns_same(ann, ann2, meta_names):
+                    print("SFFF")
+                    ann_stats.append((1, 0))
+                    are_same = False
+                else:
+                    ann_stats.append((1, 1))
+            elif not require_double_inner:
+                ann_stats.append((1, 1))
+            else:
+                ann_stats.append((1, 0))
+                are_same = False
+
+    # Check the reverse also, but only ann2 if not in first doc
+    for ann2 in document2['annotations']:
+        ann = get_ann_from_doc(document, ann2['start'], ann2['end'])
+
+        if ann is None:
+            if not require_double_inner:
+                ann_stats.append((1, 1))
+                # Also append this annotation to the document, because it is missing from it
+                document['annotations'].append(ann2)
+            else:
+                ann_stats.append((0, 1))
+                are_same = False
+
+    return are_same
+
+
+def count_annotations(data_path):
+    data = load_data(data_path, require_annotations=True)
+
+    g_cnt = 0
+    for project in data['projects']:
+        cnt = 0
+        for doc in project['documents']:
+            for ann in doc['annotations']:
+                # Only validated
+                if ann['validated']:
+                    cnt += 1
+                    g_cnt += 1
+        print("Number of documents in '{}' is: {}".format(project['name'], cnt))
+
+    print("Total number of documents is: {}".format(g_cnt))
+
+def consolidate_double_annotations(data_path, out_path, require_double=True, require_double_inner=True, meta_anns_to_match=[]):
+    """
+
+    require_double (boolean):
+        If True everything must be double annotated, meaning there have to be two projects of the same name for each name. Else, it will
+            also use projects thta do not have double annotiations.
+    require_double_inner (boolean):
+        If False - this will allow some entities to be annotated by only one annotator and not the other, while still requiring
+            annotations to be the same if they exist.
+    meta_anns_to_match (boolean):
+        List of meta annotations that must match for two annotations to be the same. If empty only the mention
+            level will be checked.
+    """
+    data = load_data(data_path, require_annotations=True)
+    out_data = {'projects': []}
+    projects_done = set()
+    ann_stats = [] # This will keep score for agreement
+    # Consolidate
+    for project in data['projects']:
+        id = project['id']
+        new_documents = []
+        if id not in projects_done:
+            projects_done.add(id)
+            name = project['name']
+            documents = project['documents']
+
+            if not require_double:
+                new_project = copy.deepcopy(project)
+                projects_done.add(id)
+            else:
+                # Means we need double annotations
+                has_double = False
+                for project2 in data['projects']:
+                    id2 = project2['id']
+                    name2 = project2['name']
+
+                    if name == name2 and id != id2:
+                        has_double = True
+                        projects_done.add(id2)
+                        break
+
+                if has_double:
+                    for document in documents:
+                        document2 = get_doc_from_project(project2, document['id'])
+
+                        if document2 is not None and are_docs_same(document, document2, require_double_inner=require_double_inner, ann_stats=ann_stats, meta_names=meta_anns_to_match):
+                            new_documents.append(document)
+
+                new_project = copy.deepcopy(project)
+                new_project['documents'] = new_documents
+            out_data['projects'].append(new_project)
+
+    # Save
+    json.dump(out_data, open(out_path, 'w'))
+
+    ann_stats = np.array(ann_stats)
+    ck = cohen_kappa_score(ann_stats[:, 0], ann_stats[:, 1])
+    t = 0
+    for i in range(len(ann_stats)):
+        if ann_stats[i, 0] == ann_stats[i, 1]:
+            t += 1
+    agr = t / len(ann_stats)
+    print("Kappa: {:.4f}; Agreement: {:.4f}".format(ck, agr)) 
+    print("Total vs InAgreement: {} / {}".format(t, len(ann_stats)))
+    return ann_stats, out_data
 
 def validate_ner_data(data_path, cdb, cntx_size=70, status_only=False, ignore_if_already_done=False):
     """ Please just ignore this function, I'm afraid to even look at it
@@ -139,6 +329,8 @@ def validate_ner_data(data_path, cdb, cntx_size=70, status_only=False, ignore_if
                             status = 'incorrect'
                         elif ann['killed']:
                             status = 'terminated'
+                        elif ann['alternative']:
+                            status = 'alternative'
                         else:
                             status = 'unk'
 
@@ -151,7 +343,11 @@ def validate_ner_data(data_path, cdb, cntx_size=70, status_only=False, ignore_if
                             print("SNIPPET           : {} <<<{}>>> {}".format(b, m, e))
                             print("CURRENT STATUS    : {}".format(status))
                             print("CURRENT ANNOTATION: {} - {}".format(cui, cdb.cui2pretty_name.get(cui, 'unk')))
+                            print("ANNS TOTAL        :")
+                            for k,v in cui2status[cui][name].items():
+                                print("        {}: {}".format(str(k), str(v)))
                             print()
+
                             d = str(input("###Change to ([q]uit/[s]kip/[c]orrect/[i]ncorrect/[t]erminate): "))
 
                             if d == 'q':
@@ -165,14 +361,17 @@ def validate_ner_data(data_path, cdb, cntx_size=70, status_only=False, ignore_if
                                 ann['correct'] = True
                                 ann['killed'] = False
                                 ann['deleted'] = False
+                                ann['alternative'] = False
                             elif d == 'i':
                                 ann['correct'] = False
                                 ann['killed'] = False
                                 ann['deleted'] = True
+                                ann['alternative'] = False
                             elif d == 't':
                                 ann['correct'] = False
                                 ann['killed'] = True
                                 ann['deleted'] = False
+                                ann['alternative'] = False
                             print()
                             print()
                 if quit:
