@@ -7,6 +7,9 @@ from torch import nn
 import torch.nn.functional as F
 import torch.optim as optim
 
+from medcat.utils.loggers import basic_logger
+log = basic_logger("utils")
+
 
 def get_batch(ind, batch_size, x, y, cpos, device):
     # Get the start/end index for this batch
@@ -24,7 +27,7 @@ def get_batch(ind, batch_size, x, y, cpos, device):
 
 def train_network(net, data, lr=0.01, test_size=0.1, max_seq_len=41, pad_id=30000, batch_size=100,
                   nepochs=20, device='cpu', save_dir='./meta_cat/', class_weights=None, ignore_cpos=False,
-                  auto_save_model=True):
+                  auto_save_model=True, score_average='weighted'):
     # Split data
     y = np.array([x[0] for x in data])
     x = [x[1] for x in data]
@@ -60,6 +63,7 @@ def train_network(net, data, lr=0.01, test_size=0.1, max_seq_len=41, pad_id=3000
     best_f1 = 0
     best_p = 0
     best_r = 0
+    best_cls_report = None
     for epoch in range(nepochs):
         running_loss_train = []
         running_loss_test = []
@@ -111,9 +115,10 @@ def train_network(net, data, lr=0.01, test_size=0.1, max_seq_len=41, pad_id=3000
         print(classification_report(y_test, np.argmax(np.concatenate(test_outs, axis=0), axis=1)))
         print("Train Loss: {:5}\nTest Loss:  {:5}\n\n".format(train_loss, test_loss))
         print("\n\n\n")
-        f1 = f1_score(y_test, np.argmax(np.concatenate(test_outs, axis=0), axis=1), average='weighted')
-        precision = precision_score(y_test, np.argmax(np.concatenate(test_outs, axis=0), axis=1), average='weighted')
-        recall = recall_score(y_test, np.argmax(np.concatenate(test_outs, axis=0), axis=1), average='weighted')
+        f1 = f1_score(y_test, np.argmax(np.concatenate(test_outs, axis=0), axis=1), average=score_average)
+        precision = precision_score(y_test, np.argmax(np.concatenate(test_outs, axis=0), axis=1), average=score_average)
+        recall = recall_score(y_test, np.argmax(np.concatenate(test_outs, axis=0), axis=1), average=score_average)
+        cls_report = classification_report(y_test, np.argmax(np.concatenate(test_outs, axis=0), axis=1), output_dict=True)
 
         if f1 > best_f1:
             print("=" * 50)
@@ -125,9 +130,55 @@ def train_network(net, data, lr=0.01, test_size=0.1, max_seq_len=41, pad_id=3000
             best_f1 = f1
             best_p = precision
             best_r = recall
+            best_cls_report = cls_report
 
             # Print some stats
             print(confusion_matrix(y_test, np.argmax(np.concatenate(test_outs, axis=0), axis=1)))
             print("\n\n")
 
-    return (best_f1, best_p, best_r)
+    return (best_f1, best_p, best_r, best_cls_report)
+
+
+def load_hf_tokenizer(tokenizer_name):
+    try:
+        from transformers import AutoTokenizer
+        hf_tokenizer = AutoTokenizer.from_pretrained(tokenizer_name)
+    except Exception as e:
+        log.exception("The Huggingface tokenizer could not be created")
+
+    return hf_tokenizer
+
+
+def build_vocab_from_hf(model_name, hf_tokenizer, vocab):
+    rebuild = False
+    # Check is it necessary
+    for i in range(hf_tokenizer.vocab_size):
+        tkn = hf_tokenizer.ids_to_tokens[i]
+        if tkn not in vocab:
+            rebuild = True
+
+    if rebuild:
+        log.info("Rebuilding vocab")
+        try:
+            from transformers import AutoModel
+            model = AutoModel.from_pretrained(model_name)
+            if 'xlnet' in model_name.lower():
+                embs = model.get_input_embeddings().weight.cpu().detach().numpy()
+            else:
+                embs = model.embeddings.word_embeddings.weight.cpu().detach().numpy()
+
+            # Reset all vecs in current vocab
+            vocab.vec_index2word = {}
+            for ind in vocab.index2word.keys():
+                vocab.vocab[vocab.index2word[ind]]['vec'] = None
+
+            for i in range(hf_tokenizer.vocab_size):
+                tkn = hf_tokenizer.ids_to_tokens[i]
+                vec = embs[i]
+                vocab.add_word(word=tkn, vec=vec, replace=True)
+
+            # Crate the new unigram table
+            vocab.reset_counts()
+            vocab.make_unigram_table()
+        except Exception as e:
+            log.exception("The Huggingface model could not be loaded")
